/*
 *	Copyright 2012  Samsung Electronics Co., Ltd
 *
 *	Licensed under the Flora License, Version 1.0 (the "License");
 *	you may not use this file except in compliance with the License.
 *	You may obtain a copy of the License at
 *
 * 		http://floralicense.org/license/
 *
 *	Unless required by applicable law or agreed to in writing, software
 *	distributed under the License is distributed on an "AS IS" BASIS,
 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *	See the License for the specific language governing permissions and
 *	limitations under the License.
 */


#include "mgr-app-common-debug.h"
#include "mgr-app-common-error.h"
#include "mgr-app-common-util.h"
#include "mgr-app-view-main.h"
#include "mgr-app-view-common.h"
#include "mgr-app-string.h"
#include "mgr-app-widget.h"
#include "mgr-app-uigadget.h"

#include <stdbool.h>
#include <pkgmgr-info.h>
#include <package-manager.h>
#include <appsvc.h>
#include <app_manager.h>


#define WRT_PREFIX 					"wgt"
#define RPM_PREFIX					"rpm"
#define TPK_PREFIX					"tpk"

#define UG_NAME_MIDLET_INFO_EFL 	"midlet-info-efl"
#define UG_NAME_WEBAPP_INFO_EFL		"webapp-detail-efl"
#define UG_PARAM_KEY_PKG_NAME		"pkg_name"

#define STR_ICON					"icon"
#define STR_UGD						"ugd"
#define STR_INSTALLED_SIZE			"installed_size"
#define STR_DATA_SIZE				"data_size"

#define STR_BTN_UNINSTALL			"btn_uninstall"
#define STR_POPUP					"popup"

#define MAX_NAME_LEN				128
#define NUM_KILO					(1024)
#define NUM_MEGA					(1024*1024)
#define NUM_GIGA					(1024*1024*1024)

#define ELM_TEXT_1					"elm.text.1"
#define ELM_TEXT_2					"elm.text.2"
#define ELM_TEXT					"elm.text"
#define ELM_ICON					"elm.icon"
#define ELM_ICON_SWALLOW_1			"elm.slide.swallow.1"
#define ELM_ICON_SWALLOW_2			"elm.slide.swallow.2"
#define ELM_BUTTON_STYLE			"popup_button/default"

#define ITC_ITEM_STYLE				"dialogue/2text.1icon.2"
#define ITC_LIFE_ITEM_STYLE			"mode/slide2/custom"
#define ITC_STO_TITLE_ITEM_STYLE	"dialogue/title"
#define ITC_STO_TOTAL_ITEM_STYLE	"dialogue/2text.2"
#define ITC_STO_APP_ITEM_STYLE 		"dialogue/2text.2"
#define ITC_STO_DATA_ITEM_STYLE 	"dialogue/2text.2"
#define ITC_DATA_ITEM_STYLE 		"dialogue/bg/2icon"
#define ITC_LAUNCH_TITLE_ITEM_STYLE	"dialogue/title"
#define ITC_LAUNCH_ITEM_STYLE 		"dialogue/1text"
#define ITC_LAUNCH_TEXT_ITEM_STYLE 	"multiline/1text"
#define ITC_DETAIL_ITEM_STYLE 		"dialogue/1text"
#define ITC_SEP_ITEM_STYLE			"dialogue/separator"

enum {
	ELM_STATE_TOP = 0,
	ELM_STATE_BOTTOM,
	ELM_STATE_CENTER,
	ELM_STATE_DEFAULT,
};

static Evas_Object *_genlist = NULL;
static mgr_app_app_info_t *_app_info = NULL;

static Elm_Genlist_Item_Class *itc = NULL;
static Elm_Genlist_Item_Class *itc_life = NULL;
static Elm_Genlist_Item_Class *itc_sto_title = NULL;
static Elm_Genlist_Item_Class *itc_sto_total = NULL;
static Elm_Genlist_Item_Class *itc_sto_app = NULL;
static Elm_Genlist_Item_Class *itc_sto_data = NULL;
static Elm_Genlist_Item_Class *itc_data = NULL;
static Elm_Genlist_Item_Class *itc_launch_title = NULL;
static Elm_Genlist_Item_Class *itc_launch = NULL;
static Elm_Genlist_Item_Class *itc_launch_text = NULL;
static Elm_Genlist_Item_Class *itc_detail = NULL;
static Elm_Genlist_Item_Class *itc_sep = NULL;

static void _free_resources(struct ug_data *ugd)
{
	MGR_APP_BEGIN();

	int ret = 0;

	if (ugd && ugd->pc) {
		MGR_APP_DEBUG("pc is freed");
		pkgmgr_client_free(ugd->pc);
		ugd->pc = NULL;
	}

	/* ug destroy */
	if (ugd && ugd->sub_ug) {
		ret = ug_destroy(ugd->sub_ug);
		if (-1 == ret) {
			MGR_APP_DEBUG_ERR("ug_destroy ugd->sub_ug is failed");
		} else {
			ugd->sub_ug = NULL;
		}
	}
	
	elm_genlist_item_class_free(itc_sep);
	elm_genlist_item_class_free(itc);
	elm_genlist_item_class_free(itc_life);
	elm_genlist_item_class_free(itc_sto_title);
	elm_genlist_item_class_free(itc_sto_total);
	elm_genlist_item_class_free(itc_sto_app);
	elm_genlist_item_class_free(itc_sto_data);
	elm_genlist_item_class_free(itc_data);
	elm_genlist_item_class_free(itc_launch_title);
	elm_genlist_item_class_free(itc_launch);
	elm_genlist_item_class_free(itc_launch_text);
	elm_genlist_item_class_free(itc_detail);

	MGR_APP_END();
}

static void _app_info_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);
	struct ug_data *ugd = (struct ug_data*)data;

	_free_resources(ugd);

	mgr_app_doubletap_sensor_disable();

	elm_naviframe_item_pop(ugd->navi_bar);

	mgrapp_set_view_type(VIEW_TYPE_APP_LIST);

	MGR_APP_END();
}

static void _force_stop_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(_app_info == NULL);
	mgr_app_app_info_t *app_info = _app_info;
	ret_if(app_info->pkg_name == NULL);
	bool is_running = false;
	int ret = APP_MANAGER_ERROR_NONE;
	struct ug_data *ugd = NULL;

	ret = app_manager_is_running(app_info->main_appid, &is_running);
	switch (ret) {
	case APP_MANAGER_ERROR_NONE:
		MGR_APP_DEBUG("is_running APP_MANAGER_ERROR_NONE");
		break;
	case APP_MANAGER_ERROR_INVALID_PARAMETER:
		MGR_APP_DEBUG("is_running APP_MANAGER_ERROR_INVALID_PARAMETER");
		return;
	default:
		MGR_APP_DEBUG("default");
		return;
	}

	if (is_running) {
		MGR_APP_DEBUG("%s app_is_running", app_info->main_appid);
		app_context_h app_context = {0, };

		ret = app_manager_get_app_context(app_info->main_appid, &app_context);
		switch (ret) {
		case APP_MANAGER_ERROR_NONE:
			MGR_APP_DEBUG("get_app_context APP_MANAGER_ERROR_NONE");
			break;
		case APP_MANAGER_ERROR_INVALID_PARAMETER:
			MGR_APP_DEBUG("get_app_context APP_MANAGER_ERROR_INVALID_PARAMETER");
			return;
		case APP_MANAGER_ERROR_DB_FAILED:
			MGR_APP_DEBUG("get_app_context APP_MANAGER_ERROR_DB_FAILED");
			return;
		case APP_MANAGER_ERROR_OUT_OF_MEMORY:
			MGR_APP_DEBUG("get_app_context APP_MANAGER_ERROR_OUT_OF_MEMORY");
			return;
		case APP_MANAGER_ERROR_NO_SUCH_APP:
			MGR_APP_DEBUG("get_app_context APP_MANAGER_ERROR_NO_SUCH_APP");
			return;
		default:
			MGR_APP_DEBUG("default");
			return;
		}

		ret = app_manager_terminate_app(app_context);
		switch (ret) {
		case APP_MANAGER_ERROR_NONE:
			MGR_APP_DEBUG("terminate_app APP_MANAGER_ERROR_NONE");
			break;
		case APP_MANAGER_ERROR_INVALID_PARAMETER:
			MGR_APP_DEBUG("terminate_app APP_MANAGER_ERROR_INVALID_PARAMETER");
			return;
		default:
			MGR_APP_DEBUG("default");
			return;
		}

		ret = app_context_destroy(app_context);
		switch (ret) {
		case APP_MANAGER_ERROR_NONE:
			MGR_APP_DEBUG("context_destroy APP_MANAGER_ERROR_NONE");
			break;
		case APP_MANAGER_ERROR_INVALID_PARAMETER:
			MGR_APP_DEBUG("context_destroy APP_MANAGER_ERROR_INVALID_PARAMETER");
			break;
		default:
			MGR_APP_DEBUG("default");
			break;
		}
		ugd = evas_object_data_get(_genlist, STR_UGD);
		mgrapp_view_list_update_by_uninstall(app_info->pkg_type, app_info->pkg_name, (void*)ugd);

	} else {
		MGR_APP_DEBUG("%s app is not running", app_info->main_appid);
	}


	if (obj) {
		elm_object_disabled_set(obj, EINA_TRUE);
	}

	MGR_APP_END();
}

static void _no_response_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);
	evas_object_del(data);

	MGR_APP_END();
}

static void _uninstall_by_pkgmgr_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(obj == NULL);
	ret_if(_app_info == NULL);
	mgr_app_app_info_t *app_info = _app_info;

	struct ug_data *ugd = NULL;
	ugd		= evas_object_data_get(_genlist, STR_UGD);
	ret_if(ugd == NULL);

	Evas_Object *btn_uninstall = NULL;
	btn_uninstall = (Evas_Object*)evas_object_data_get(obj, STR_BTN_UNINSTALL);
	ret_if(btn_uninstall == NULL);

	Evas_Object *popup = NULL;
	popup = (Evas_Object*)evas_object_data_get(obj, STR_POPUP);
	ret_if(popup == NULL);

	evas_object_data_del(obj, STR_BTN_UNINSTALL);
	evas_object_data_del(obj, STR_POPUP);
	evas_object_del(popup);

	mgr_app_create_popup_progressbar(ugd->win_main, MGRAPP_STR_UNINSTALLING);

	_force_stop_clicked_cb(data, btn_uninstall, event_info);

	MGR_APP_DEBUG("pkg_type: %s, pkg_name: %s", app_info->pkg_type, app_info->pkg_name);
	int req_id = 0;
	if (ugd->pc) {
		pkgmgr_client_free(ugd->pc);
		ugd->pc = NULL;
	}
	ugd->pc = pkgmgr_client_new(PC_REQUEST);
	if (!ugd->pc) {
		MGR_APP_DEBUG_ERR("create new pkgmgr client is failed");
		MGR_APP_END();
		return;
	}

	req_id = pkgmgr_client_uninstall(ugd->pc, app_info->pkg_type, app_info->pkg_name, PM_QUIET, /*mgrapp_view_listen_cb*/NULL, /*(void*)ugd*/NULL);

	if (req_id > 0) {
		MGR_APP_DEBUG("uninstall success, req_id=%d", req_id);
	} else {
		switch (req_id) {
		case PKGMGR_R_EINVAL:	/* -3: Invalid argument */
			MGR_APP_DEBUG("PKGMGR_R_EINVAL");
			break;
		case PKGMGR_R_ECOMM:	/* -2: Comunication Error */
			MGR_APP_DEBUG("PKGMGR_R_ECOMM");
			break;
		case PKGMGR_R_OK:		/* 0: General success */
			MGR_APP_DEBUG("PKGMGR_R_OK");
			break;
		default:
			MGR_APP_DEBUG("default req_id=%d", req_id);
			break;
		}
	}

	/* the select 'No' noti be available from pkg-mgr then add the condition */
	elm_object_disabled_set(obj, EINA_FALSE);

	MGR_APP_END();
}

static void _uninstall_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(obj == NULL);
	Evas_Object *btn_uninstall = obj;

	ret_if(_app_info == NULL);

	Evas_Object *popup = NULL;
	Evas_Object *btn_yes = NULL;
	Evas_Object *btn_no = NULL;
	char buf[MAX_NAME_LEN] = {0, };
	struct ug_data *ugd = NULL;

	ugd = evas_object_data_get(_genlist, STR_UGD);
	ret_if(ugd == NULL);

	popup = elm_popup_add(ugd->navi_bar);
	ret_if(popup == NULL);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	snprintf(buf, sizeof(buf), "%s?", MGRAPP_STR_UNINSTALL);
	elm_object_text_set(popup, buf);
	elm_object_part_text_set(popup, "title,text", _app_info->pkg_label);

	btn_yes = elm_button_add(popup);
	ret_if(btn_yes == NULL);
	elm_object_text_set(btn_yes, MGRAPP_STR_YES);
	elm_object_part_content_set(popup, "button1", btn_yes);
	elm_object_style_set(btn_yes, ELM_BUTTON_STYLE);
	evas_object_smart_callback_add(btn_yes, "clicked", _uninstall_by_pkgmgr_cb, data);
	evas_object_data_set(btn_yes, STR_BTN_UNINSTALL, btn_uninstall);
	evas_object_data_set(btn_yes, STR_POPUP, popup);

	btn_no = elm_button_add(popup);
	ret_if(btn_no == NULL);
	elm_object_text_set(btn_no, MGRAPP_STR_NO);
	elm_object_part_content_set(popup, "button2", btn_no);
	elm_object_style_set(btn_no, ELM_BUTTON_STYLE);
	evas_object_smart_callback_add(btn_no, "clicked", _no_response_cb, (void*)popup);

	evas_object_show(popup);

	MGR_APP_END();
}

static void _clear_data_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();
	/* TODO */
	if (obj) {
		elm_object_disabled_set(obj, EINA_TRUE);
	}
	MGR_APP_END();
}

static void _gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	Elm_Object_Item *item = (Elm_Object_Item *)event_info;

	if (item) {
		elm_genlist_item_selected_set(item, EINA_FALSE);
	}

	MGR_APP_END();
}

static void _gl_sel_launch(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	Elm_Object_Item *item = (Elm_Object_Item *)event_info;
	ret_if(data == NULL);
	struct ug_data *ugd = (struct ug_data*)data;

	ret_if(_app_info == NULL);
	mgr_app_app_info_t *app_info = _app_info;

	ret_if(app_info->pkg_name == NULL);
	char *package_name = NULL;
	package_name = app_info->pkg_name;

	if (item) {
		elm_genlist_item_selected_set(item, EINA_FALSE);
		elm_object_item_disabled_set(item, EINA_TRUE);
	}

	if (appsvc_is_defapp(package_name)) {
		MGR_APP_DEBUG("%s is the default app", package_name);
		if (appsvc_unset_defapp(package_name) != APPSVC_RET_OK) {
			MGR_APP_DEBUG_ERR("unset the %s to default app is failed", package_name);
		}
	} else {
		MGR_APP_DEBUG("%s is not the default app", package_name);
	}

	MGR_APP_END();
}

static void _layout_cb(ui_gadget_h ug, enum ug_mode mode, void *priv)
{
	MGR_APP_BEGIN();

	ret_if(NULL == priv);

	Evas_Object *base = NULL;

	base = ug_get_layout(ug);
	ret_if(NULL == base);

	switch (mode) {
	case UG_MODE_FULLVIEW:
		evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		ug_disable_effect(ug);
		evas_object_show(base);
		break;
	default:
		break;
	}

	MGR_APP_END();
}

static void _result_cb(ui_gadget_h ug, service_h service, void *priv)
{
	MGR_APP_BEGIN();

	ret_if(NULL == priv);
	
	struct ug_data *mydata = NULL;
	char *val = NULL;

	mydata = priv;
	if (service) {
		int ret = SERVICE_ERROR_NONE;

		ret = service_get_extra_data(service, "name", &val);
		switch (ret) {
		case SERVICE_ERROR_NONE:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_NONE");
			break;
		case SERVICE_ERROR_INVALID_PARAMETER:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_INVALID_PARAMETER");
			break;
		case SERVICE_ERROR_KEY_NOT_FOUND:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_KEY_NOT_FOUND");
			break;
		case SERVICE_ERROR_OUT_OF_MEMORY:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_OUT_OF_MEMORY");
			break;
		case SERVICE_ERROR_INVALID_DATA_TYPE:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_INVALID_DATA_TYPE");
			break;
		default:
			MGR_APP_DEBUG_ERR("ret value has an error. %d.", ret);
			break;
		}

		if (val) {
			MGR_APP_DEBUG("The name of UI gadget that sends result is [%s]", val);
		}
	}

	ug_destroy(ug);

	MGR_APP_DEBUG("mydata->sub_ug:%d", mydata->sub_ug);

	mydata->sub_ug = NULL;

	MGR_APP_END();
}

static void _destroy_cb(ui_gadget_h ug, void *priv)
{
	MGR_APP_BEGIN();

	ret_if(NULL == ug);
	ret_if(NULL == priv);

	struct ug_data *mydata = (struct ug_data*)priv;

	ug_destroy(ug);
	mydata->sub_ug = NULL;

	MGR_APP_END();
}

static void _create_ug(void *data)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);
	struct ug_data *ugd = (struct ug_data*)data;
	mgr_app_app_info_t *app_info = NULL;

	ret_if(ugd->data == NULL);
	app_info = (mgr_app_app_info_t*)ugd->data;

	ui_gadget_h ug = {0, };
	struct ug_cbs cbs = {0,};
	service_h service = {0, };
	int ret = MGR_APP_OK;

	cbs.layout_cb 	= _layout_cb;
	cbs.result_cb 	= _result_cb;
	cbs.destroy_cb 	= _destroy_cb;
	cbs.priv		= (void*)ugd;

	ret = service_create(&service);
	switch (ret) {
	case SERVICE_ERROR_NONE:
		MGR_APP_DEBUG("service_create SERVICE_ERROR_NONE");
		break;
	case SERVICE_ERROR_INVALID_PARAMETER:
		MGR_APP_DEBUG_ERR("service_create SERVICE_ERROR_INVALID_PARAMETER");
		return;
	case SERVICE_ERROR_OUT_OF_MEMORY:
		MGR_APP_DEBUG_ERR("service_create SERVICE_ERROR_OUT_OF_MEMORY");
		return;
	default:
		MGR_APP_DEBUG_ERR("service_create ret value has an error, %d.", ret);
		return;
	}

	if (app_info->pkg_type && 0 == strncmp(app_info->pkg_type, WRT_PREFIX, strlen(WRT_PREFIX))) {
		MGR_APP_DEBUG("app_info->pkg_type: %s", app_info->pkg_type);
		MGR_APP_DEBUG("app_info->pkg_name : %s", app_info->pkg_name);

		ret = service_add_extra_data(service, UG_PARAM_KEY_PKG_NAME, app_info->pkg_name);
		switch (ret) {
		case SERVICE_ERROR_NONE:
			MGR_APP_DEBUG("service_add_extra_data SERVICE_ERROR_NONE");
			break;
		case SERVICE_ERROR_INVALID_PARAMETER:
			MGR_APP_DEBUG_ERR("service_add_extra_data SERVICE_ERROR_INVALID_PARAMETER");
			goto end;
		case SERVICE_ERROR_KEY_REJECTED:
			MGR_APP_DEBUG_ERR("service_add_extra_data SERVICE_ERROR_KEY_REJECTED");
			goto end;
		default:
			MGR_APP_DEBUG_ERR("service_add_extra_data ret value has an error, %d.", ret);
			goto end;
		}

		/* wrt ug more detail info */
		ug = ug_create(ugd->ug, UG_NAME_WEBAPP_INFO_EFL, UG_MODE_FULLVIEW, service, &cbs);
		ugd->sub_ug = ug;
	} else {
		MGR_APP_DEBUG(">>> NOT SUPPORTED TYPE");
	}

end:
	ret = service_destroy(service);
	switch (ret) {
	case SERVICE_ERROR_NONE:
		MGR_APP_DEBUG("service_destroy SERVICE_ERROR_NONE");
		break;
	case SERVICE_ERROR_INVALID_PARAMETER:
		MGR_APP_DEBUG_ERR("service_destroy SERVICE_ERROR_INVALID_PARAMETER");
		break;
	case SERVICE_ERROR_OUT_OF_MEMORY:
		MGR_APP_DEBUG_ERR("service_destroy SERVICE_ERROR_OUT_OF_MEMORY");
		break;
	default:
		MGR_APP_DEBUG_ERR("service_destroy ret value has an error, %d.", ret);
		break;
	}

	MGR_APP_END();
}

static void _gl_sel_detail(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	Elm_Object_Item *item = (Elm_Object_Item *)event_info;
	struct ug_data *ugd = (struct ug_data*)data;

	if (item) {
		elm_genlist_item_selected_set(item, 0);
	}

	_create_ug((void*)ugd);

	MGR_APP_END();
}

static char *_get_formated_size(float size)
{
	MGR_APP_BEGIN();

	char *buf = (char*)malloc(sizeof(char)*MAX_NAME_LEN);
	retv_if(buf == NULL, NULL);
	memset(buf, 0, MAX_NAME_LEN);

	if (size < NUM_KILO) {
		snprintf(buf, MAX_NAME_LEN, "%d %s", (int)size, MGRAPP_STR_BYTES);
	} else if (size >= NUM_KILO && size < NUM_MEGA) {
		snprintf(buf, MAX_NAME_LEN, "%'.2f %s", size / NUM_KILO, MGRAPP_STR_KB);
	} else if (size >= NUM_MEGA && size < NUM_GIGA) {
		snprintf(buf, MAX_NAME_LEN, "%'.2f %s", size / NUM_MEGA, MGRAPP_STR_MB);
	} else {
		snprintf(buf, MAX_NAME_LEN, "%'.2f %s", size / NUM_GIGA, MGRAPP_STR_GB);
	}

	MGR_APP_END();
	return buf;
}

static char *_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	retv_if(_app_info == NULL, NULL);
	mgr_app_app_info_t *app_info = _app_info;

	char *label = NULL;
	char buf[MAX_NAME_LEN] = {0, };

	char *package_name	= app_info->pkg_name;
	char *package_label	= app_info->pkg_label;
	//char *app_name 	= app_info->app_name;
	char *icon_path 	= app_info->icon_path;
	char *pkg_type		= app_info->pkg_type;

//	MGR_APP_DEBUG("package_name:[%s], app_name:[%s], x_slp_packagetype:[%s], icon_path:[%s]\n", package_name, app_name, pkg_type, icon_path);
	MGR_APP_DEBUG("package_label:[%s], x_slp_packagetype:[%s], icon_path:[%s]\n", package_label, pkg_type, icon_path);

	MGR_APP_DEBUG("part:%s\n", part);
	retv_if(package_label == NULL, NULL);

	/* label for 'elm.text' part */
	if (0 == strcmp(part, ELM_TEXT_1)) {
		if (package_label) {
			MGR_APP_MEM_STRDUP(label, package_label);
		}
	} else if (0 == strcmp(part, ELM_TEXT_2)) {
		snprintf(buf, MAX_NAME_LEN, "%s: %s", MGRAPP_STR_VERSION, app_info->pkg_version);
		MGR_APP_MEM_STRDUP(label, buf);
	} else {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_ERROR);
	}

	MGR_APP_END();
	return label;
}

static char *_gl_label_get_sto_total(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	retv_if(_app_info == NULL, NULL);
	mgr_app_app_info_t *app_info = _app_info;

	char *label = NULL;

	MGR_APP_DEBUG("part:%s\n", part);

	/* label for 'elm.text' part */
	if (0 == strcmp(part, ELM_TEXT_2)) {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_TOTAL);
	} else if (0 == strcmp(part, ELM_TEXT_1)) {
		label = _get_formated_size((float)app_info->total_size);
	} else {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_ERROR);
	}

	MGR_APP_END();
	return label;
}

static char *_gl_label_get_sto_app(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	retv_if(_app_info == NULL, NULL);
	mgr_app_app_info_t *app_info = _app_info;

	char *label = NULL;

	MGR_APP_DEBUG("part:%s\n", part);

	/* label for 'elm.text' part */
	if (0 == strcmp(part, ELM_TEXT_2)) {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_APPLICATION);
	} else if (0 == strcmp(part, ELM_TEXT_1)) {
		label = _get_formated_size((float)app_info->total_size - (float)app_info->data_size);
	} else {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_ERROR);
	}

	MGR_APP_END();
	return label;
}

static char *_gl_label_get_sto_data(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	retv_if(_app_info == NULL, NULL);
	mgr_app_app_info_t *app_info = _app_info;

	char *label = NULL;

	MGR_APP_DEBUG("part:%s\n", part);

	/* label for 'elm.text' part */
	if (0 == strcmp(part, ELM_TEXT_2)) {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_DATA);
	} else if (0 == strcmp(part, ELM_TEXT_1)) {
		label = _get_formated_size((float)app_info->data_size);
	} else {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_ERROR);
	}

	MGR_APP_END();
	return label;
}

static char *_gl_label_get_launch(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	char *label = NULL;

	MGR_APP_DEBUG("part:%s\n", part);

	/* label for 'elm.text' part */
	if (0 == strcmp(part, ELM_TEXT)) {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_CLEAR_DEFAULTS);
	}

	MGR_APP_END();
	return label;
}

static char *_gl_label_get_detail(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	char *label = NULL;

	MGR_APP_DEBUG("part:%s\n", part);

	/* label for 'elm.text' part */
	if (0 == strcmp(part, ELM_TEXT)) {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_MORE_DETAILS);
	}

	MGR_APP_END();
	return label;
}

static char *_gl_label_get_launch_text(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	char *label = NULL;

	MGR_APP_DEBUG("part:%s\n", part);

	/* label for 'elm.text' part */
	if (0 == strcmp(part, ELM_TEXT)) {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_DEFAULT_DESC);
	}

	MGR_APP_END();
	return label;
}

static Evas_Object *_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	retv_if(_app_info == NULL, NULL);
	mgr_app_app_info_t *app_info = _app_info;

	Evas_Object *icon = NULL;
	char *icon_path = NULL;

	retv_if(app_info->icon_path == NULL, NULL);
	icon_path = app_info->icon_path;

	MGR_APP_DEBUG("icon_path:[%s]\n", icon_path);
	MGR_APP_DEBUG("part:%s\n", part);

	/* icon for 'elm.icon' part */
	icon = elm_icon_add(obj);
	retv_if(icon == NULL, NULL);
	elm_image_file_set(icon, icon_path, STR_ICON);

	evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);

	MGR_APP_END();

	return icon;
}

static Evas_Object *_gl_icon_get_life(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	MGR_APP_DEBUG("part:%s\n", part);
	retv_if(_app_info == NULL, NULL);
	mgr_app_app_info_t *app_info = _app_info;

	Evas_Object *button = NULL;
	if (0 == strcmp(part, ELM_ICON_SWALLOW_2)) {
		button = elm_button_add(obj);
		if (button) {
			elm_object_style_set(button, ELM_BUTTON_STYLE);
			elm_object_disabled_set(button, EINA_TRUE);
			elm_object_text_set(button, MGRAPP_STR_UNINSTALL);
			evas_object_smart_callback_add(button, "clicked", _uninstall_clicked_cb, (void*)data);

			if (app_info->removable) {
				elm_object_disabled_set(button, EINA_FALSE);
			}
		}
	} else if (0 == strcmp(part, ELM_ICON_SWALLOW_1)) {
		button = elm_button_add(obj);
		if (button) {
			elm_object_style_set(button, ELM_BUTTON_STYLE);
			elm_object_disabled_set(button, EINA_TRUE);
			elm_object_text_set(button, MGRAPP_STR_FORCE_STOP);
			evas_object_smart_callback_add(button, "clicked", _force_stop_clicked_cb, (void*)data);

			if (app_info->taskmanage) {
				bool is_running = false;
				int ret = app_manager_is_running(app_info->main_appid, &is_running);

				switch (ret) {
				case APP_MANAGER_ERROR_NONE:
					MGR_APP_DEBUG("is_running APP_MANAGER_ERROR_NONE");
					if (is_running) {
						elm_object_disabled_set(button, EINA_FALSE);
					}
					break;
				case APP_MANAGER_ERROR_INVALID_PARAMETER:
					MGR_APP_DEBUG("is_running APP_MANAGER_ERROR_INVALID_PARAMETER");
					break;
				default:
					MGR_APP_DEBUG("default");
					break;
				}
			}
		}
	} else {
		MGR_APP_DEBUG_ERR("elm icon part is wrong");
		button = NULL;
	}

	MGR_APP_END();
	return button;
}

static Evas_Object *_gl_icon_get_data(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	MGR_APP_DEBUG("part:%s\n", part);
	retv_if(_app_info == NULL, NULL);
	mgr_app_app_info_t *app_info = _app_info;

	Evas_Object *button = NULL;
	if (0 == strcmp(part, ELM_ICON)) {
		button = elm_button_add(obj);
		if (button) {
			elm_object_text_set(button, MGRAPP_STR_CLEAR_DATA);
			elm_object_style_set(button, ELM_BUTTON_STYLE);
			evas_object_smart_callback_add(button, "clicked", _clear_data_clicked_cb, (void*)data);
			if (app_info->data_size <= 0) {
				elm_object_disabled_set(button, EINA_TRUE);
			}
		}
	} else {
		MGR_APP_DEBUG_ERR("empty button area");
		button = NULL;
	}

	MGR_APP_END();
	return button;
}

static char *_gl_label_get_title(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	char *group_title = (char *)data;
	char *title = NULL;

	MGR_APP_MEM_STRDUP(title, group_title);
	MGR_APP_END();

	return title;
}

static void _gl_realized(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(event_info == NULL);

	int type = (int)elm_object_item_data_get(event_info);

	if (type == ELM_STATE_TOP) {
		elm_object_item_signal_emit(event_info, "elm,state,top", "");
	} else if (type == ELM_STATE_CENTER) {
		elm_object_item_signal_emit(event_info, "elm,state,center", "");
	} else if (type == ELM_STATE_BOTTOM) {
		elm_object_item_signal_emit(event_info, "elm,state,bottom", "");
	}

	MGR_APP_END();
}

static Evas_Object *_create_genlist_app_info(Evas_Object *parent, void *data)
{
	MGR_APP_BEGIN();

	retv_if(data == NULL, NULL);
	struct ug_data *ugd = (struct ug_data*)data;

	Elm_Object_Item	*item_app			= NULL;
	Elm_Object_Item	*item_life			= NULL;
	Elm_Object_Item	*item_sto_total	= NULL;
	Elm_Object_Item	*item_sto_app		= NULL;
	Elm_Object_Item	*item_sto_data		= NULL;
	Elm_Object_Item	*item_data			= NULL;
	Elm_Object_Item	*item_launch_text	= NULL;
	Elm_Object_Item	*item_launch		= NULL;
	Elm_Object_Item	*item_sep		= NULL;
	Elm_Object_Item	*item_detail		= NULL;

	Evas_Object	*genlist 	= NULL;
	char *sto_title 		= NULL;
	char *launch_title		= NULL;


	/* set the genlist item style */
	itc_sep = elm_genlist_item_class_new();
	retv_if(itc_sep == NULL, NULL);
	itc = elm_genlist_item_class_new();
	retv_if(itc == NULL, NULL);
	itc_life = elm_genlist_item_class_new();
	retv_if(itc_life == NULL, NULL);
	itc_sto_title = elm_genlist_item_class_new();
	retv_if(itc_sto_title == NULL, NULL);
	itc_sto_total = elm_genlist_item_class_new();
	retv_if(itc_sto_total == NULL, NULL);
	itc_sto_app = elm_genlist_item_class_new();
	retv_if(itc_sto_app == NULL, NULL);
	itc_sto_data = elm_genlist_item_class_new();
	retv_if(itc_sto_data == NULL, NULL);
	itc_data = elm_genlist_item_class_new();
	retv_if(itc_data == NULL, NULL);
	itc_launch_title = elm_genlist_item_class_new();
	retv_if(itc_launch_title == NULL, NULL);
	itc_launch = elm_genlist_item_class_new();
	retv_if(itc_launch == NULL, NULL);
	itc_launch_text = elm_genlist_item_class_new();
	retv_if(itc_launch_text == NULL, NULL);
	itc_detail = elm_genlist_item_class_new();
	retv_if(itc_detail == NULL, NULL);

	itc_sep->item_style			= ITC_SEP_ITEM_STYLE;
	itc_sep->func.text_get 		= NULL;
	itc_sep->func.content_get	= NULL;
	itc_sep->func.state_get		= NULL;
	itc_sep->func.del			= NULL;

	itc->item_style 			= ITC_ITEM_STYLE;
	itc->func.text_get 			= _gl_label_get;
	itc->func.content_get 		= _gl_icon_get;
	itc->func.state_get			= NULL;
	itc->func.del 				= NULL;

	itc_life->item_style 		= ITC_LIFE_ITEM_STYLE;
	itc_life->func.text_get		= NULL;
	itc_life->func.content_get 	= _gl_icon_get_life;
	itc_life->func.state_get	= NULL;
	itc_life->func.del 			= NULL;

	itc_sto_title->item_style 		= ITC_STO_TITLE_ITEM_STYLE;
	itc_sto_title->func.text_get	= _gl_label_get_title;
	itc_sto_title->func.content_get = NULL;
	itc_sto_title->func.state_get 	= NULL;
	itc_sto_title->func.del 		= NULL;

	itc_sto_total->item_style 		= ITC_STO_TOTAL_ITEM_STYLE;
	itc_sto_total->func.text_get 	= _gl_label_get_sto_total;
	itc_sto_total->func.content_get = NULL;
	itc_sto_total->func.state_get 	= NULL;
	itc_sto_total->func.del 		= NULL;

	itc_sto_app->item_style 		= ITC_STO_APP_ITEM_STYLE;
	itc_sto_app->func.text_get 		= _gl_label_get_sto_app;
	itc_sto_app->func.content_get 	= NULL;
	itc_sto_app->func.state_get 	= NULL;
	itc_sto_app->func.del 			= NULL;

	itc_sto_data->item_style 		= ITC_STO_DATA_ITEM_STYLE;
	itc_sto_data->func.text_get		= _gl_label_get_sto_data;
	itc_sto_data->func.content_get	= NULL;
	itc_sto_data->func.state_get	= NULL;
	itc_sto_data->func.del 			= NULL;

	itc_data->item_style 			= ITC_DATA_ITEM_STYLE;
	itc_data->func.text_get			= NULL;
	itc_data->func.content_get		= _gl_icon_get_data;
	itc_data->func.state_get		= NULL;
	itc_data->func.del 				= NULL;

	itc_launch_title->item_style 	= ITC_LAUNCH_TITLE_ITEM_STYLE;
	itc_launch_title->func.text_get	= _gl_label_get_title;
	itc_launch_title->func.content_get 	= NULL;
	itc_launch_title->func.state_get	= NULL;
	itc_launch_title->func.del 		= NULL;

	itc_launch->item_style 			= ITC_LAUNCH_ITEM_STYLE;
	itc_launch->func.text_get 		= _gl_label_get_launch;
	itc_launch->func.content_get 	= NULL;
	itc_launch->func.state_get 		= NULL;
	itc_launch->func.del 			= NULL;

	itc_launch_text->item_style 	= ITC_LAUNCH_TEXT_ITEM_STYLE;
	itc_launch_text->func.text_get 	= _gl_label_get_launch_text;
	itc_launch_text->func.content_get 	= NULL;
	itc_launch_text->func.state_get = NULL;
	itc_launch_text->func.del 		= NULL;

	itc_detail->item_style 			= ITC_DETAIL_ITEM_STYLE;
	itc_detail->func.text_get 		= _gl_label_get_detail;
	itc_detail->func.content_get 	= NULL;
	itc_detail->func.state_get 		= NULL;
	itc_detail->func.del 			= NULL;

	retv_if(ugd->data == NULL, NULL);
	mgr_app_app_info_t *app_info = (mgr_app_app_info_t *)ugd->data;

	/* add a genlist */
	genlist = elm_genlist_add(parent);
	retv_if(genlist == NULL, NULL);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	elm_scroller_policy_set(genlist, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
	evas_object_data_set(genlist, STR_UGD, (void*)ugd);
	evas_object_show(genlist);
	evas_object_smart_callback_add(genlist, "realized", _gl_realized, NULL);

	MGR_APP_MEM_STRDUP(sto_title, MGRAPP_STR_STORAGE);
	MGR_APP_MEM_STRDUP(launch_title, MGRAPP_STR_LAUNCH_BY_DEFAULT);


	/* app info */
	item_app = elm_genlist_item_append(genlist, itc, ELM_STATE_DEFAULT, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_app)
		elm_genlist_item_select_mode_set(item_app, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

	/* Force stop & Uninstall */
	item_life = elm_genlist_item_append(genlist, itc_life, ELM_STATE_DEFAULT, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_life)
		elm_genlist_item_select_mode_set(item_life, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

	/* Storage */
	elm_genlist_item_append(genlist, itc_sto_title, sto_title, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
	item_sto_total = elm_genlist_item_append(genlist, itc_sto_total, ELM_STATE_TOP, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sto_total)
		elm_genlist_item_select_mode_set(item_sto_total, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	item_sto_app = elm_genlist_item_append(genlist, itc_sto_app, ELM_STATE_CENTER, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sto_app)
		elm_genlist_item_select_mode_set(item_sto_app, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	item_sto_data = elm_genlist_item_append(genlist, itc_sto_data, ELM_STATE_BOTTOM, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sto_data)
		elm_genlist_item_select_mode_set(item_sto_data, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);


	/* separator with no bottom line */
	item_sep = elm_genlist_item_append(genlist, itc_sep, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sep)
		elm_genlist_item_select_mode_set(item_sep, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

#if 0
	/* Clear Data */
	/* remove the button until the feature is supported */
	item_data = elm_genlist_item_append(genlist, &itc_data, (void *)app_info, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel, NULL);


	/* separator with bottom line */
	/* the separator is located bottom of Clear Data button */
	item_sep = elm_genlist_item_append(genlist, &itc_sep, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sep)
		elm_genlist_item_select_mode_set(item_sep, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
#endif

	/* Launch */
	elm_genlist_item_append(genlist, itc_launch_title, launch_title, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
	item_launch = elm_genlist_item_append(genlist, itc_launch, ELM_STATE_DEFAULT, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel_launch, (void*)ugd);
	if (item_launch && !appsvc_is_defapp(app_info->pkg_name)) {
		elm_genlist_item_selected_set(item_launch, EINA_FALSE);
		elm_object_item_disabled_set(item_launch, EINA_TRUE);
	}
	item_launch_text = elm_genlist_item_append(genlist, itc_launch_text, ELM_STATE_DEFAULT, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_launch_text)
		elm_genlist_item_select_mode_set(item_launch_text, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	item_sep = elm_genlist_item_append(genlist, itc_sep, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sep)
		elm_genlist_item_select_mode_set(item_sep, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);


	/* More Detail */
	item_detail = elm_genlist_item_append(genlist, itc_detail, ELM_STATE_DEFAULT, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel_detail, (void*)ugd);
	if (item_detail) {
		elm_genlist_item_selected_set(item_detail, EINA_FALSE);
		elm_object_item_disabled_set(item_detail, EINA_TRUE);
	}
#if 0
	if (app_info->pkg_type) {
		if (item_detail && (!strncmp(app_info->pkg_type, TPK_PREFIX, strlen(TPK_PREFIX)) || !strncmp(app_info->pkg_type, RPM_PREFIX, strlen(RPM_PREFIX)))) {
			elm_genlist_item_selected_set(item_detail, EINA_FALSE);
			elm_object_item_disabled_set(item_detail, EINA_TRUE);
		}
	} else {
		MGR_APP_DEBUG_ERR("app_info->pkg_type is NULL");
	}
#endif

	/* footer border */
	item_sep = elm_genlist_item_append(genlist, itc_sep, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sep)
		elm_genlist_item_select_mode_set(item_sep, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

	if (elm_genlist_first_item_get(genlist)) {
		item_sep = elm_genlist_item_prepend(genlist, itc_sep, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		if (item_sep)
			elm_genlist_item_select_mode_set(item_sep, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}

	MGR_APP_END();
	return genlist;
}

static void _doubletap_sensor_cb(void *data)
{
	MGR_APP_BEGIN();

	ret_if(NULL == data);

	Evas_Object *genlist = (Evas_Object*)data;
	Elm_Object_Item *item = NULL;

	item = elm_genlist_first_item_get(genlist);
	ret_if(NULL == item);
	
	elm_genlist_item_show(item, ELM_GENLIST_ITEM_SCROLLTO_TOP);
		
	MGR_APP_END();
}

static void _get_app_info_from_pkgmgr(char *pkg_name, mgr_app_app_info_t *app_info)
{
	MGR_APP_BEGIN();

	ret_if(NULL == pkg_name);
	
	pkgmgrinfo_appinfo_h appinfo_h = NULL;
	pkgmgrinfo_pkginfo_h pkginfo_h = NULL;
	char *main_appid = NULL;
	char *icon_path = NULL;	
	char *pkg_label = NULL;
	char *pkg_type = NULL;
	char *pkg_version = NULL;
	char *data_size = NULL;
	char *total_size = NULL;
	int r = 0;

	MGR_APP_DEBUG("pkg_name: %s", pkg_name);
	MGR_APP_MEM_STRDUP(app_info->pkg_name, pkg_name);

	r = pkgmgrinfo_pkginfo_get_pkginfo(pkg_name, &pkginfo_h);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_pkginfo is failed %d", r);
		goto end;
	}
	MGR_APP_DEBUG("pkg_name: %s", pkg_name);

	r = pkgmgrinfo_pkginfo_get_mainappid(pkginfo_h, &main_appid);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_mainappid is failed %d", r);
		goto end;
	}
	MGR_APP_DEBUG("main_appid: %s", main_appid);

	r = pkgmgrinfo_appinfo_get_appinfo(main_appid, &appinfo_h);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_get_appinfo is failed %d", r);
		goto end;
	}

	if (NULL == app_info->pkg_label) {
		r = pkgmgrinfo_appinfo_get_label(appinfo_h, &pkg_label);
		if (r < 0) {
			MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_get_label failed. %d", r);
			goto end;
		} else if (pkg_label) {
			MGR_APP_MEM_STRDUP(app_info->pkg_label, pkg_label);
		} else {
			MGR_APP_MEM_STRDUP(app_info->pkg_label, "");
		}
	} 
	MGR_APP_DEBUG("pkg_label: %s", pkg_label);

	if (NULL == app_info->pkg_type) {
		r = pkgmgrinfo_pkginfo_get_type(pkginfo_h, &pkg_type);
		if (r < 0) {
			MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_type failed. %d", r);
			goto end;
		} else if (pkg_type) {
			MGR_APP_MEM_STRDUP(app_info->pkg_type, pkg_type);
		} else {
			MGR_APP_MEM_STRDUP(app_info->pkg_type, "");
		}
	} 
	MGR_APP_DEBUG("pkg_type: %s", pkg_type);

	if (NULL == app_info->pkg_version) {
		r = pkgmgrinfo_pkginfo_get_version(pkginfo_h, &pkg_version);
		if (r < 0) {
			MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_version failed. %d", r);
			goto end;
		} else if (pkg_version) {
			MGR_APP_MEM_STRDUP(app_info->pkg_version, pkg_version);
		} else {
			MGR_APP_MEM_STRDUP(app_info->pkg_version, "");
		}
	} 
	MGR_APP_DEBUG("pkg_version: %s", pkg_version);

	r = pkgmgrinfo_pkginfo_get_data_size(pkginfo_h, &app_info->data_size);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_data_size failed. %d", r);
		goto end;
	} 
	MGR_APP_DEBUG("data_size: %d", data_size);
	
	r = pkgmgrinfo_pkginfo_get_total_size(pkginfo_h, &app_info->total_size);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_total_size failed. %d", r);
		goto end;
	} 
	MGR_APP_DEBUG("total_size: %d", total_size);

	r = pkgmgrinfo_pkginfo_is_removable(pkginfo_h, &app_info->removable);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_is_removable failed. %d", r);
		goto end;
	} 

	r = pkgmgrinfo_appinfo_is_taskmanage(appinfo_h, &app_info->taskmanage);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_is_taskmanage failed. %d", r);
		goto end;
	} 

	r = pkgmgrinfo_appinfo_is_nodisplay(appinfo_h, &app_info->nodisplay);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_is_nodisplay failed. %d", r);
		goto end;
	} 

	if (NULL == app_info->icon_path) {
		r = pkgmgrinfo_appinfo_get_icon(appinfo_h, &icon_path);
		if (r < 0) {
			MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_get_icon failed. %d", r);
			goto end;
		} else if (icon_path) {
			MGR_APP_MEM_STRDUP(app_info->icon_path, icon_path);
		} else {
			MGR_APP_MEM_STRDUP(app_info->icon_path, "");
		}

		if (app_info->nodisplay) {
			mgr_app_set_default_icon(&app_info->icon_path, DEFAULT_ICON_SERVICE);
		} else {
			mgr_app_set_default_icon(&app_info->icon_path, DEFAULT_ICON_APP);
		}
	} 
	MGR_APP_DEBUG("icon_path: %s", icon_path);

end:
	if (appinfo_h) {
		pkgmgrinfo_appinfo_destroy_appinfo(appinfo_h);
	}

	if (pkginfo_h) {
		pkgmgrinfo_pkginfo_destroy_pkginfo(pkginfo_h);
	}
	
	MGR_APP_END();
}


void mgrapp_view_info_create(char *pkg_name, void *data)
{
	MGR_APP_BEGIN();

	ret_if(NULL == pkg_name || NULL == data);
	struct ug_data *ugd = (struct ug_data*)data;
	Evas_Object *back_btn = NULL;

	mgrapp_set_view_type(VIEW_TYPE_APP_INFO);

	if (ugd->data) {
		_app_info = (mgr_app_app_info_t*)ugd->data;
	} else {
		_app_info = (mgr_app_app_info_t*)malloc(sizeof(mgr_app_app_info_t));
		ret_if(_app_info == NULL);
		memset(_app_info, 0, sizeof(mgr_app_app_info_t));
		ugd->data = (void*)_app_info;
	}

	_get_app_info_from_pkgmgr(pkg_name, _app_info);
	
	_genlist = _create_genlist_app_info(ugd->navi_bar, (void*)ugd);

	/* back button */
	back_btn = elm_button_add(ugd->navi_bar);
	ret_if(NULL == back_btn);
	elm_object_style_set(back_btn, "naviframe/end_btn/default");
	evas_object_smart_callback_add(back_btn, "clicked", _app_info_back_cb, (void*)ugd);

	elm_naviframe_item_push(ugd->navi_bar, MGRAPP_STR_APPLICATION_INFO, back_btn, NULL, _genlist, NULL);

	mgr_app_doubletap_sensor_set_cb(_doubletap_sensor_cb, _genlist);
	mgr_app_doubletap_sensor_enable();

	MGR_APP_END();
}

void mgrapp_view_info_update(void *data)
{
	MGR_APP_BEGIN();

	if (_genlist) {
		elm_genlist_realized_items_update(_genlist);
	}

	MGR_APP_END();
}

void mgrapp_view_info_destroy(void *data)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);
	struct ug_data *ugd = (struct ug_data*)data;

	_free_resources(ugd);

	mgr_app_doubletap_sensor_disable();

	elm_naviframe_item_pop(ugd->navi_bar);

	mgrapp_set_view_type(VIEW_TYPE_APP_LIST);

#if 0
	/* for Always ug */
	/* bg must delete before starting on_destroy */
	if (ugd->bg) {
		evas_object_del(ugd->bg);
		ugd->bg = NULL;
	}

	if (ugd->ug) {
		ug_destroy_me(ugd->ug);
		ugd->ug = NULL;
	}
#endif

	MGR_APP_END();
}
