/*
 *	Copyright 2013  Samsung Electronics Co., Ltd
 *
 *	Licensed under the Flora License, Version 1.0 (the "License");
 *	you may not use this file except in compliance with the License.
 *	You may obtain a copy of the License at
 *
 * 		http://floralicense.org/license/
 *
 *	Unless required by applicable law or agreed to in writing, software
 *	distributed under the License is distributed on an "AS IS" BASIS,
 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *	See the License for the specific language governing permissions and
 *	limitations under the License.
 */


#include "mgr-app-view-common.h"
#include "mgr-app-common-debug.h"
#include "mgr-app-common-util.h"

#include <sensors.h>

#define STR_PNG				".png"
#define STR_EDJ				".edj"

static sensor_h _dt_handle = NULL;
static bool _is_sensor_enabled = false;
static MGR_APP_DOUBLETAP_SENSOR_CB _sensor_cb = NULL;

static view_type_t _view = VIEW_TYPE_NONE;
static const char *_icon_file[DEFAULT_ICON_MAX] = {
	IMGDIR"/ug-setting-manage-applications-efl/default_icon_app.png",
	IMGDIR"/ug-setting-manage-applications-efl/default_icon_service.png"
};

void mgrapp_set_view_type(view_type_t view)
{
	MGR_APP_BEGIN();
	_view = view;
	MGR_APP_END();
}

view_type_t mgrapp_get_view_type()
{
	MGR_APP_BEGIN();
	MGR_APP_END();
	return _view;
}

static void _doubletap_sensor_cb(unsigned long long timestamp, void *user_data)
{
	MGR_APP_BEGIN();

	if (_sensor_cb) {
		(_sensor_cb)(user_data);
	}
	
	MGR_APP_END();
}

void mgr_app_doubletap_sensor_set_cb(MGR_APP_DOUBLETAP_SENSOR_CB sensor_cb, void *user_data)
{
	MGR_APP_BEGIN();

	ret_if(NULL == sensor_cb);

	_sensor_cb = sensor_cb;
	
	if (NULL == _dt_handle) {
		if (sensor_create(&_dt_handle)) {
			MGR_APP_DEBUG_ERR("FAILED: sensor_create");
		}
	}

	sensor_motion_doubletap_unset_cb(_dt_handle);

	if(!sensor_motion_doubletap_set_cb(_dt_handle, _doubletap_sensor_cb, user_data)) {
		MGR_APP_DEBUG("SUCCESS: sensor_motion_shake_set_cb");
	} else {
		MGR_APP_DEBUG_ERR("FAILED: sensor_motion_shake_set_cb");
	}

	MGR_APP_END();
}

void mgr_app_doubletap_sensor_enable()
{
	MGR_APP_BEGIN();

	ret_if(NULL == _sensor_cb);
	
	if (_dt_handle && false == _is_sensor_enabled
			&& !sensor_start(_dt_handle, SENSOR_MOTION_DOUBLETAP)) {
		_is_sensor_enabled = true;
		MGR_APP_DEBUG("SUCCESS: sensor_start");
	} else {
		MGR_APP_DEBUG_ERR("FAILED: sensor_start");
	}
	
	MGR_APP_END();
}

void mgr_app_doubletap_sensor_disable()
{
	MGR_APP_BEGIN();

	if (_dt_handle && true == _is_sensor_enabled) {
		if (!sensor_stop(_dt_handle, SENSOR_MOTION_DOUBLETAP)) {
			_is_sensor_enabled = false;
			MGR_APP_DEBUG("SUCCESS: sensor_stop");
		} else {
			MGR_APP_DEBUG_ERR("FAILED: sensor_stop");
		}
	} 

	MGR_APP_END();
}

void mgr_app_doubletap_sensor_destroy()
{
	MGR_APP_BEGIN();

	sensor_motion_doubletap_unset_cb(_dt_handle);

	if (_dt_handle) {
		sensor_destroy(_dt_handle);
		_dt_handle = NULL;
	}
	_is_sensor_enabled = false;
	
	MGR_APP_END();
}

void mgr_app_set_default_icon(char **icon_path, icon_t type)
{
	MGR_APP_BEGIN();

	FILE *stream = NULL;
	char *p = NULL;

	if (type < DEFAULT_ICON_APP || DEFAULT_ICON_MAX <= type) {
		MGR_APP_DEBUG_ERR("icon type is out of bound. %d", type);
		return;
	}
	
	if (*icon_path) {
		/* TODO - avoid the slow-down change effect due to edj icon */
		if (((p = strrchr(*icon_path, '.'))) && (!strcasecmp(p, ".edj"))) {
			MGR_APP_MEM_FREE(*icon_path);
			MGR_APP_MEM_STRDUP(*icon_path, _icon_file[type]);
		} else if ((stream = fopen(*icon_path, "r")) != NULL && strlen(*icon_path) > strlen(STR_PNG)) {
			fclose(stream);
			stream = NULL;

			char *p_extension = NULL;
			p_extension = *icon_path + (strlen(*icon_path) - strlen(STR_PNG));

			/* default icon : icon_path's extension is wrong */
			if (strncmp(p_extension, STR_PNG, strlen(STR_PNG)) != 0 && strncmp(p_extension, STR_EDJ, strlen(STR_EDJ)) != 0) {
				MGR_APP_MEM_FREE(*icon_path);
				MGR_APP_MEM_STRDUP(*icon_path, _icon_file[type]);
			}
		/* default icon : icon_path isn't exist */
		} else {
			MGR_APP_MEM_FREE(*icon_path);
			MGR_APP_MEM_STRDUP(*icon_path, _icon_file[type]);
			if (stream) {
				fclose(stream);
				stream = NULL;
			}
		}
	/* default icon : icon_path == NULL */
	} else {
		MGR_APP_MEM_FREE(*icon_path);
		MGR_APP_MEM_STRDUP(*icon_path, _icon_file[type]);
	}
	
	MGR_APP_END();
}

