/*
 *	Copyright 2012  Samsung Electronics Co., Ltd
 *
 *	Licensed under the Flora License, Version 1.0 (the "License");
 *	you may not use this file except in compliance with the License.
 *	You may obtain a copy of the License at
 *
 * 		http://floralicense.org/license/
 *
 *	Unless required by applicable law or agreed to in writing, software
 *	distributed under the License is distributed on an "AS IS" BASIS,
 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *	See the License for the specific language governing permissions and
 *	limitations under the License.
 */


#include "mgr-app-widget.h"
#include "mgr-app-common-debug.h"
#include "mgr-app-common-util.h"

static Evas_Object *_popup = NULL;

Evas_Object *mgr_app_view_create_base_navigation(Evas_Object *parent)
{
	MGR_APP_BEGIN();
	retv_if(parent == NULL, NULL);

	Evas_Object *navibar = NULL;

	navibar = elm_naviframe_add(parent);
	elm_object_part_content_set(parent, "elm.swallow.content", navibar);

	evas_object_show(navibar);

	MGR_APP_END();
	return navibar;
}

Evas_Object *mgr_app_widget_create_bg(Evas_Object *parent)
{
	MGR_APP_BEGIN();
	retv_if(parent == NULL, NULL);

	Evas_Object *bg = NULL;

	bg = elm_bg_add(parent);
    evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(bg);

	MGR_APP_END();
	return bg;
}

Evas_Object *mgr_app_widget_create_main_layout(Evas_Object *parent)
{
	MGR_APP_BEGIN();
	retv_if(parent == NULL, NULL);

	Evas_Object *layout = NULL;

	layout = elm_layout_add(parent);
	elm_layout_theme_set(layout, "layout", "application", "default");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(layout);

	MGR_APP_END();
	return layout;
}

Evas_Object *mgr_app_create_layout(Evas_Object *parent, const char *clas, const char *group, const char *style)
{
	MGR_APP_BEGIN();
	retv_if(parent == NULL, NULL);

	Evas_Object *ly = NULL;

	ly = elm_layout_add(parent);
	elm_layout_theme_set(ly, clas, group, style);
	evas_object_size_hint_weight_set(ly, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ly, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(ly);
	MGR_APP_END();

	return ly;
}

Evas_Object *mgr_app_create_layout_file(Evas_Object *parent, const char *filename, const char *group)
{
	MGR_APP_BEGIN();
	retv_if(parent == NULL, NULL);

	Evas_Object *ly = NULL;

	ly = elm_layout_add(parent);
	elm_layout_file_set(ly, filename, group);
	evas_object_size_hint_weight_set(ly, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ly, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(ly);
	MGR_APP_END();

	return ly;
}

Evas_Object *mgr_app_create_button(Evas_Object *parent, void (*func) (void *data, Evas_Object *obj, void *event_info), const char *label, const char *swallow, const char *style, void *data)
{
	MGR_APP_BEGIN();
	retv_if(parent == NULL, NULL);

	Evas_Object *button = NULL;

	button = elm_button_add(parent);
	elm_object_part_content_set(parent, swallow, button);
	elm_object_style_set(button, style);
	elm_object_text_set(button, label);
	evas_object_show(button);
	evas_object_smart_callback_add(button, "clicked", func, (void*)data);
	MGR_APP_END();
	return button;
}

Evas_Object *mgr_app_create_label(Evas_Object *parent, const char *text, const char *swallow, void *data)
{
	MGR_APP_BEGIN();
	retv_if(parent == NULL, NULL);

	Evas_Object *label = NULL;

	label = elm_label_add(parent);
	elm_object_part_content_set(parent, swallow, label);
	elm_object_text_set(label, text);
	MGR_APP_END();

	return label;
}

void mgrapp_create_popup_text_1button(Evas_Object *parent, char *text, char *btn_text, Evas_Smart_Cb func)
{
	MGR_APP_BEGIN();

	ret_if(parent == NULL);
	ret_if(text == NULL);
	ret_if(btn_text == NULL);

	Evas_Object *popup = NULL;
	Evas_Object *btn = NULL;

	popup = elm_popup_add(parent);
	ret_if(popup == NULL);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(popup, text);

	btn = elm_button_add(popup);
	ret_if(btn == NULL);
	elm_object_text_set(btn, btn_text);
	elm_object_part_content_set(popup, "button1", btn);
	evas_object_smart_callback_add(btn, "clicked", func, popup);
	evas_object_show(popup);

	MGR_APP_BEGIN();
}

void mgr_app_create_popup_progressbar(Evas_Object *parent, char *text)
{
	MGR_APP_BEGIN();

	ret_if(parent == NULL);
	ret_if(text == NULL);

	Evas_Object *progressbar = NULL;

	if (_popup) {
		evas_object_del(_popup);
		_popup = NULL;
	}

	_popup = elm_popup_add(parent);
	ret_if(_popup == NULL);

	progressbar = elm_progressbar_add(_popup);
	ret_if(progressbar == NULL);
	elm_object_style_set(progressbar, "list_process");
	evas_object_size_hint_align_set(progressbar, EVAS_HINT_FILL, 0.5);
	evas_object_size_hint_weight_set(progressbar, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_progressbar_pulse(progressbar, EINA_TRUE);
	evas_object_show(progressbar);

	elm_object_part_text_set(_popup, "title,text", text);
	elm_object_content_set(_popup, progressbar);
	evas_object_show(_popup);

	MGR_APP_END();
}

void mgrapp_destroy_popup_progressbar()
{
	MGR_APP_BEGIN();

	if (_popup) {
		evas_object_del(_popup);
		_popup = NULL;
	}

	MGR_APP_END();
}
